@@ -65,7 +65,7 @@ uint32_t (*convert_endian_l)(uint32_t);
 uint16_t (*convert_endian_s)(uint16_t);
 
 static int iLogLevel = 1;
-char hpFileName[] = "/tmp/hplipfaxLog_XXXXXX";
+char hpFileName[64] = "/tmp/hplipfaxLog_XXXXXX";
 
 #define TIFF_HDR_SIZE 8
 #define LITTLE_ENDIAN_MODE I
@@ -439,14 +439,20 @@ int ProcessTiffData(int fromFD, int toFD)
     int bytes_written = 0;
     int ret_status = 0;
     int bytes_read = 0;
-    char hpTiffFileName[] = "/tmp/hpliptiffXXXXXX";
+    char hpTiffFileName[64] = "/tmp/hpliptiffXXXXXX";
     long input_file_size = 0;
 
     fdTiff = mkstemp (hpTiffFileName);
     if (fdTiff < 0)
     {
-        BUG ("ERROR: Unable to open Fax output file - %s for writing\n", hpTiffFileName);
-        return 1;
+        DBG ("Warning: Unable to open Fax output file - %s for writing, so trying another location\n", hpTiffFileName);
+        strncpy(hpTiffFileName,"/var/log/hp/hpliptiffXXXXXX",sizeof(hpTiffFileName));
+        fdTiff = mkstemp (hpTiffFileName);
+        if (fdTiff < 0)
+        {
+            BUG("ERROR: Unable to open Fax output file - %s for writing\n", hpTiffFileName);
+            return 1;
+        }
     }
 
     memset (szFileHeader, 0, sizeof (szFileHeader));
@@ -593,6 +599,10 @@ int ProcessTiffData(int fromFD, int toFD)
     HPLIPPUTINT32 ((szFileHeader + 9), page_counter);
     write (toFD, szFileHeader + 9, 4);
 
+	if (!(iLogLevel & SAVE_PCL_FILE))
+	{
+         unlink(hpTiffFileName);
+	}
     return ret_status;
 }
 
@@ -667,11 +677,18 @@ int main (int argc, char **argv)
          i++;
     }
 
+
     fdFax = mkstemp (hpFileName);
     if (fdFax < 0)
     {
-        BUG ("ERROR: Unable to open Fax output file - %s for writing\n", hpFileName);
-        return 1;
+        DBG ("Warning: Unable to open Fax output file - %s for writing,so trying another location.\n", hpFileName);
+        strncpy(hpFileName,"/var/log/hp/hplipfaxLog_XXXXXX",sizeof(hpFileName));
+        fdFax = mkstemp (hpFileName);
+        if (fdFax < 0)
+        {
+            BUG ("ERROR: Unable to open Fax output file - %s for writing\n", hpFileName);
+            return 1;
+        }
     }
 
     /*********** MAIN ***********/
