@@ -375,7 +375,10 @@ DRIVER_ERROR UXServices::ToDevice(const BYTE * pBuffer, DWORD * Count)
     {
         fwrite (pBuffer, 1, *Count, outfp);
 	if (!(m_iLogLevel & SEND_TO_PRINTER))
+	{
+	    *Count = 0;
 	    return NO_ERROR;
+	}
     }
 
    /* Write must be not-buffered, don't use streams */
@@ -415,6 +418,15 @@ DRIVER_ERROR UXServices::ReadDeviceID (BYTE * strID, int iSize)
    return NO_ERROR;
 }
 
+#ifdef HP_PRINTVIEW
+const char *szPJLHeader = "@PJL SET JOBATTR=\"JobAcct7=HPPrintView.exe\"\012";
+int UXServices::GetPJLHeaderBuffer (char **szPJLBuffer)
+{
+    *szPJLBuffer = (char *) szPJLHeader;
+    return strlen (szPJLHeader);
+}
+#endif // HP_PRINTVIEW
+
 BOOL UXServices::GetVerticalAlignmentValue(BYTE* cVertAlignVal)
 {
    if (VertAlign == -1)
