@@ -699,6 +699,11 @@ SANE_Status soapht_control_option(SANE_Handle handle, SANE_Int option, SANE_Acti
                   break;
                }
             }
+            if (stat != SANE_STATUS_GOOD)
+            {
+                ps->currentResolution = ps->resolutionList[1];
+                stat = SANE_STATUS_GOOD;
+            }
          }
          else
          {  /* Set default. */
@@ -912,7 +917,11 @@ SANE_Status soapht_start(SANE_Handle handle)
    int stat, ret;
 
    DBG8("sane_hpaio_start()\n");
-
+   
+    ps -> user_cancel = 0;
+    ps -> cnt = 0;
+    ps -> index = 0;
+  
    if (set_extents(ps))
    {
       BUG("invalid extents: tlx=%d brx=%d tly=%d bry=%d minwidth=%d minheight%d maxwidth=%d maxheight=%d\n",
@@ -928,6 +937,7 @@ SANE_Status soapht_start(SANE_Handle handle)
       if (ret == 0)
       {
          stat = SANE_STATUS_NO_DOCS;     /* done scanning */
+         SendScanEvent (ps->uri, EVENT_SCAN_ADF_NO_DOCS);
          goto bugout;
       }
       else if (ret < 0)
@@ -943,7 +953,7 @@ SANE_Status soapht_start(SANE_Handle handle)
       stat = SANE_STATUS_IO_ERROR;
       goto bugout;
    }
-
+   SendScanEvent(ps->uri, EVENT_START_SCAN_JOB);
    memset(xforms, 0, sizeof(xforms));    
 
    /* Setup image-processing pipeline for xform. */
@@ -1081,7 +1091,13 @@ SANE_Status soapht_read(SANE_Handle handle, SANE_Byte *data, SANE_Int maxLength,
    int ret, stat=SANE_STATUS_IO_ERROR;
 
    DBG8("sane_hpaio_read() handle=%p data=%p maxLength=%d\n", (void *)handle, data, maxLength);
-
+   if(ps->user_cancel)
+   {
+     DBG8("soapht_read() EVENT_SCAN_CANCEL****uri=%s\n", ps->uri);
+     SendScanEvent(ps->uri, EVENT_SCAN_CANCEL);
+     return SANE_STATUS_CANCELLED;
+   }
+  
    ret = get_ip_data(ps, data, maxLength, length);
 
    if(ret & (IP_INPUT_ERROR | IP_FATAL_ERROR))
@@ -1091,7 +1107,10 @@ SANE_Status soapht_read(SANE_Handle handle, SANE_Byte *data, SANE_Int maxLength,
    }
 
    if (ret & IP_DONE)
+   {
       stat = SANE_STATUS_EOF;
+      SendScanEvent(ps->uri, EVENT_END_SCAN_JOB);
+   }
    else
       stat = SANE_STATUS_GOOD;
 
@@ -1122,7 +1141,7 @@ void soapht_cancel(SANE_Handle handle)
     * Sane_cancel is always called at the end of the scan job. Note that on a multiple page scan job 
     * sane_cancel is called only once.
     */
-
+   ps -> user_cancel = 1;
    if (ps->ip_handle)
    {
       ipClose(ps->ip_handle); 
@@ -1131,7 +1150,3 @@ void soapht_cancel(SANE_Handle handle)
    ps->bb_end_scan(ps, 0);
 } /* soapht_cancel */
 
-
-
-
-
