@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Authors: Don Welch
+# Authors: Don Welch, Naga Samrat Chowdary Narla,
 #
 
 # StdLib
@@ -85,7 +85,7 @@ class AlignDialog(QDialog, Ui_Dialog):
             ALIGN_TYPE_UNSUPPORTED : 1,
             ALIGN_TYPE_AUTO : 2,
             ALIGN_TYPE_9XX : 7,
-            ALIGN_TYPE_8XX : 6,
+            ALIGN_TYPE_8XX : 7,
             ALIGN_TYPE_LIDIL_0_3_8 : 0,
             ALIGN_TYPE_LIDIL_0_4_3 : 0,
             ALIGN_TYPE_LIDIL_AIO : 3,
@@ -97,6 +97,10 @@ class AlignDialog(QDialog, Ui_Dialog):
             ALIGN_TYPE_OJ_PRO : 0,
             ALIGN_TYPE_TEST : 0,
             ALIGN_TYPE_AIO : 3,
+            ALIGN_TYPE_LIDIL_DJ_D1600: 0,
+            ALIGN_TYPE_LEDM: 0,
+            ALIGN_TYPE_LEDM_MANUAL: 0,
+            ALIGN_TYPE_LEDM_FF_CC_0: 0,
             }
 
         self.seq = { # (func|method, tuple of params|None)
@@ -118,7 +122,7 @@ class AlignDialog(QDialog, Ui_Dialog):
                                 (self.endFronPanelPage, None),
                                 (self.close, None),
                             ],
-            
+
 #            ALIGN_TYPE_NONE : [ # 0
 #                               (self.close, None)
 #                            ],
@@ -126,6 +130,7 @@ class AlignDialog(QDialog, Ui_Dialog):
             ALIGN_TYPE_AUTO : [ # 1
                                (self.showLoadPaperPage, None),
                                (maint.AlignType1PML, (lambda : self.dev, lambda: true)),
+                               (self.closeAll, None),
                                (self.close, None),
                             ],
 
@@ -139,10 +144,10 @@ class AlignDialog(QDialog, Ui_Dialog):
                                 (self.endAlignmentNumberPage, ('B',)),
                                 (self.showAlignmentNumberPage, ('C', 'v', 'kc', 2, 5)),
                                 (self.endAlignmentNumberPage, ('C',)),
-                                (self.setAlignButton, (BUTTON_ALIGN,)),
                                 (self.showAlignmentNumberPage, ('D', 'v', 'c', 2, 5)),
                                 (self.endAlignmentNumberPage, ('D',)),
-                                (self.showLoadPaperPage, None),
+                                (self.setAlignButton, (BUTTON_ALIGN,)),
+                                (self.showLoadPaperPage, (lambda: True,)),
                                 (maint.alignType2Phase2, (lambda: self.dev, lambda: self.a, lambda: self.b,
                                                           lambda: self.c, lambda: self.d)),
                                 (self.close, None),
@@ -302,7 +307,7 @@ class AlignDialog(QDialog, Ui_Dialog):
                                (maint.alignType10Phase2, (lambda: self.dev, lambda: self.values,
                                                           lambda: self.pattern)),
                                (self.setAlignButton, (BUTTON_FINISH,)),
-                               (self.showLoadPaperPage, None),
+                               (self.showLoadPaperPage, (lambda: True,)),
                                (maint.alignType10Phase3, (lambda: self.dev,)),
                                (self.close, None),
                             ],
@@ -314,9 +319,9 @@ class AlignDialog(QDialog, Ui_Dialog):
                                (self.showLBowPage, (lambda: self.pattern,)),
                                (self.endLBowPage, None), # sets values
                                (maint.alignType11Phase2, (lambda: self.dev, lambda: self.values,
-                                                          lambda: self.pattern)),
+                                                          lambda: self.pattern, lambda: self.dev.pen_config)),
                                (self.setAlignButton, (BUTTON_FINISH,)),
-                               (self.showLoadPaperPage, None),
+                               (self.showLoadPaperPage, (lambda: True,)),
                                (maint.alignType11Phase3, (lambda: self.dev,)),
                                (self.close, None),
                             ],
@@ -335,7 +340,58 @@ class AlignDialog(QDialog, Ui_Dialog):
                               (self.close, None),
                             ],
 
+            ALIGN_TYPE_LIDIL_DJ_D1600 : [ # 14
+                               (self.showLoadPaperPage, None),
+                               (maint.alignType14Phase1, (lambda: self.dev,)),
+                               (self.setAlignButton, (BUTTON_ALIGN,)),
+                               (self.showLBowPage, (lambda: self.pattern,)),
+                               (self.endLBowPage, None), # sets values
+                               (maint.alignType14Phase2, (lambda: self.dev, lambda: self.values,
+                                                          lambda: self.pattern, lambda: self.dev.pen_config)),
+                               (self.setAlignButton, (BUTTON_FINISH,)),
+                               (self.showLoadPaperPage, (lambda: True,)),
+                               (maint.alignType14Phase3, (lambda: self.dev,)),
+                               (self.close, None),
+                            ],
 
+            ALIGN_TYPE_LEDM : [ # 15
+                               (self.showLoadPaperPage, None),
+                               (maint.AlignType15Phase1, (lambda : self.dev, lambda: self.showAioPage)),
+                               (self.close, None),
+                            ],
+
+            ALIGN_TYPE_LEDM_MANUAL : [ # 16
+                               (self.showLoadPaperPage, None),
+                               (maint.AlignType15Phase1, (lambda : self.dev, lambda: true)),
+                               (self.showAlignmentNumberPage, ('A', 'v', 'kc', 3, 23)),
+                               (self.endAlignmentNumberPage, ('A',)),
+                               (self.showAlignmentNumberPage, ('B', 'h', 'kc', 3, 11)),
+                               (self.endAlignmentNumberPage, ('B',)),
+                               (self.showAlignmentNumberPage, ('C', 'v', 'k', 3, 23)),
+                               (self.endAlignmentNumberPage, ('C',)),
+                               (self.showAlignmentNumberPage, ('D', 'v', 'c', 3, 23)),
+                               (self.endAlignmentNumberPage, ('D',)),
+                               (self.showAlignmentNumberPage, ('E', 'h', 'k', 3, 11)),
+                               (self.endAlignmentNumberPage, ('E',)),
+                               (self.showAlignmentNumberPage, ('F', 'h', 'k', 3, 11)),
+                               (self.endAlignmentNumberPage, ('F',)),
+                               (self.showAlignmentNumberPage, ('G', 'h', 'k', 3, 11)),
+                               (self.endAlignmentNumberPage, ('G',)),
+                               (self.showAlignmentNumberPage, ('H', 'v', 'k', 3, 9)),
+                               (self.endAlignmentNumberPage, ('H',)),
+                               (self.showAlignmentNumberPage, ('I', 'v', 'c', 3, 9)),
+                               (self.endAlignmentNumberPage, ('I',)),
+                               (maint.AlignType16Phase1, (lambda: self.dev, lambda: self.a, lambda: self.b,
+                                                          lambda: self.c, lambda: self.d, lambda: self.e,
+                                                          lambda: self.f, lambda: self.g, lambda: self.h,
+                                                          lambda: self.i)),
+                               (self.close, None),
+                            ],
+           ALIGN_TYPE_LEDM_FF_CC_0 : [ # 17
+                               (self.showLoadPaperPage, None),
+                               (maint.AlignType17Phase1, (lambda : self.dev, lambda: self.showAioPage)),
+                               (self.close, None),
+                            ],
             }
 
         self.setupUi(self)
@@ -354,7 +410,7 @@ class AlignDialog(QDialog, Ui_Dialog):
         self.DeviceComboBox.setFilter({'align-type': (operator.gt, 0)})
 
         # Application icon
-        self.setWindowIcon(QIcon(load_pixmap('prog', '48x48')))
+        self.setWindowIcon(QIcon(load_pixmap('hp_logo', '128x128')))
 
         if self.device_uri:
             self.DeviceComboBox.setInitialDevice(self.device_uri)
@@ -420,7 +476,7 @@ class AlignDialog(QDialog, Ui_Dialog):
 
 
     def showStartPage(self):
-        self.BackButton.setEnabled(False)
+#       self.BackButton.setEnabled(False)
         num_devices = self.DeviceComboBox.setDevices()
 
         if num_devices == 1:
@@ -448,7 +504,9 @@ class AlignDialog(QDialog, Ui_Dialog):
             return
 
 
-    def showLoadPaperPage(self):
+    def showLoadPaperPage(self, finish=False):
+        if finish:
+            self.LoadPaper.button_name = self.__tr("Finish >")
         self.LoadPaper.updateUi()
         self.displayPage(PAGE_LOAD_PAPER)
 
@@ -463,7 +521,7 @@ class AlignDialog(QDialog, Ui_Dialog):
         # colors: 'k' or 'c' or 'kc'
         # line_count: 2 or 3
         # choice_count: 5, 7, 9, 11, etc. (odd)
-        self.AlignmentNumberTitle.setText(self.__tr("Choose the set of lines in group %1 where the line segments are <b>best</b> aligned.").arg(line_id))
+        self.AlignmentNumberTitle.setText(self.__tr("From the printed Alignment page, Choose the set of lines in group %1 where the line segments are <b>best</b> aligned.").arg(line_id))
         self.AlignmentNumberIcon.setPixmap(load_pixmap('%s-%s-%d' % (orientation, colors, line_count), 'other'))
         self.AlignmentNumberComboBox.clear()
 
@@ -471,6 +529,7 @@ class AlignDialog(QDialog, Ui_Dialog):
             self.AlignmentNumberComboBox.addItem(QString("%1%2").arg(line_id).arg(x+1))
 
         self.displayPage(PAGE_ALIGNMENT_NUMBER)
+        return
 
 
     def endAlignmentNumberPage(self, line_id):
@@ -492,6 +551,25 @@ class AlignDialog(QDialog, Ui_Dialog):
             self.d = v
             log.debug("D=%d" % v)
 
+        elif line_id == 'E':
+            self.e = v
+            log.debug("E=%d" % v)
+
+        elif line_id == 'F':
+            self.f = v
+            log.debug("F=%d" % v)
+
+        elif line_id == 'G':
+            self.g = v
+            log.debug("G=%d" % v)
+
+        elif line_id == 'H':
+            self.h = v
+            log.debug("H=%d" % v)
+
+        elif line_id == 'I':
+            self.i = v
+            log.debug("I=%d" % v)
 
     def showPageEdgePage(self, prefix=None, count=13):
         self.PageEdgeTitle.setText(self.__tr("Choose the <b>numbered arrow</b> that <b>best </b>marks the edge of the paper."))
@@ -519,6 +597,9 @@ class AlignDialog(QDialog, Ui_Dialog):
         if self.align_type == ALIGN_TYPE_LBOW:
             pattern = maint.alignType10SetPattern(self.dev)
 
+        elif self.align_type == ALIGN_TYPE_LIDIL_DJ_D1600:
+            pattern = maint.alignType14SetPattern(self.dev)
+
         else: # ALIGN_TYPE_LIDIL_0_5_4
             pattern = maint.alignType11SetPattern(self.dev)
 
@@ -526,10 +607,11 @@ class AlignDialog(QDialog, Ui_Dialog):
             log.error("Invalid pattern!")
             # TODO: ...
 
-        self.controls = maint.align10and11Controls(pattern, self.align_type)
-
+        self.controls = maint.align10and11and14Controls(pattern, self.align_type)
+        keys = self.controls.keys()
+        keys.sort()
         max_line = 'A'
-        for line in self.controls:
+        for line in keys:
             if self.controls[line][0]:
                 max_line = line
             else:
@@ -539,7 +621,7 @@ class AlignDialog(QDialog, Ui_Dialog):
 
         for line in self.controls:
             if not self.controls[line][0]:
-                eval('self.%sComboBox%s.setEnabled(False)' % line.lower())
+                eval('self.%sComboBox.setEnabled(False)' % line.lower())
             else:
                 for x in range(self.controls[line][1]):
                     eval('self.%sComboBox.addItem("%s%d")' % (line.lower(), line, x + 1))
@@ -555,7 +637,7 @@ class AlignDialog(QDialog, Ui_Dialog):
 
         for line in controls:
             if not self.controls[line][0]:
-                ret.append(0)
+                self.values.append(0)
             else:
                 exec('selected = unicode(self.%sComboBox.currentText())' % line.lower())
                 try:
@@ -599,11 +681,11 @@ class AlignDialog(QDialog, Ui_Dialog):
 
 
     def showFrontPanelPage(self):
-        self.BackButton.setEnabled(False)
+#       self.BackButton.setEnabled(False)
         self.setAlignButton(BUTTON_FINISH)
         self.displayPage(PAGE_FRONT_PANEL)
-        
-        
+
+
     def endFronPanelPage(self):
         pass
 
@@ -677,5 +759,10 @@ class AlignDialog(QDialog, Ui_Dialog):
         self.dev.pen_config = status.getPenConfiguration(self.dev.getStatusFromDeviceID())
 
 
+    def closeAll(self):
+        if self.dev is not None:
+            self.dev.close()
+
+
     def __tr(self,s,c = None):
         return qApp.translate("AlignDialog",s,c)
