@@ -31,7 +31,8 @@ from prnt import cups
 from base.codes import *
 from ui_utils import *
 #from installer import core_install
-from installer.core_install import CoreInstall
+from installer.core_install import CoreInstall,\
+                                   MODE_CHECK
 
 # Qt
 from PyQt4.QtCore import *
@@ -77,6 +78,8 @@ DEVICE_DESC_MULTI_FUNC = 2
 class PasswordDialog(QDialog):
     def __init__(self, prompt, parent=None, name=None, modal=0, fl=0):
         QDialog.__init__(self, parent)
+        # Application icon
+        self.setWindowIcon(QIcon(load_pixmap('hp_logo', '128x128')))
         self.prompt = prompt
 
         Layout= QGridLayout(self)
@@ -110,7 +113,12 @@ class PasswordDialog(QDialog):
         self.connect(self.OkPushButton, SIGNAL("clicked()"), self.accept)
         self.connect(self.PasswordLineEdit, SIGNAL("returnPressed()"), self.accept)
 
-
+    def setDefaultUsername(self, defUser, allowUsernameEdit = True):
+        self.UsernameLineEdit.setText(defUser)
+        if not allowUsernameEdit:
+            self.UsernameLineEdit.setReadOnly(True)
+            self.UsernameLineEdit.setStyleSheet("QLineEdit {background-color: lightgray}")
+    
     def getUsername(self):
         return unicode(self.UsernameLineEdit.text())
 
@@ -132,10 +140,13 @@ class PasswordDialog(QDialog):
 
 
 
-def showPasswordUI(prompt):
-    try:
+def showPasswordUI(prompt, userName=None, allowUsernameEdit=True):
+    try:	
         dlg = PasswordDialog(prompt, None)
 
+        if userName != None:
+            dlg.setDefaultUsername(userName, allowUsernameEdit)
+
         if dlg.exec_() == QDialog.Accepted:
             return (dlg.getUsername(), dlg.getPassword())
 
@@ -641,17 +652,17 @@ class SetupDialog(QDialog, Ui_Dialog):
 
     def showAddPrinterPage(self):
         # Install the plugin if needed...
-        core = CoreInstall()
+        core = CoreInstall(MODE_CHECK)
         plugin = self.mq.get('plugin', PLUGIN_NONE)
         plugin_reason = self.mq.get('plugin-reason', PLUGIN_REASON_NONE)
         if plugin > PLUGIN_NONE:
 
-            if not core.check_for_plugin():
+            if core.check_for_plugin() != PLUGIN_INSTALLED:
                 ok, sudo_ok = pkit.run_plugin_command(plugin == PLUGIN_REQUIRED, plugin_reason)
                 if not sudo_ok:
                     FailureUI(self, self.__tr("<b>Unable to find an appropriate su/sudo utiltity to run hp-plugin.</b><p>Install kdesu, gnomesu, or gksu.</p>"))
                     return
-                if not ok or not core.check_for_plugin():
+                if not ok or core.check_for_plugin() != PLUGIN_INSTALLED:
                     if plugin == PLUGIN_REQUIRED:
                         FailureUI(self, self.__tr("<b>The printer you are trying to setup requires a binary driver plug-in and it failed to install.</b><p>Please check your internet connection and try again.</p><p>Visit <u>http://hplipopensource.com</u> for more infomation.</p>"))
                         return
@@ -729,8 +740,18 @@ class SetupDialog(QDialog, Ui_Dialog):
         try:
             self.print_ppd = None
             self.ppds = cups.getSystemPPDs()
-            model = cups.stripModel2(self.model)
-            self.print_ppd = cups.getPPDFile2(model, self.ppds)
+            
+            #Check if common ppd name is already given in models.dat(This is needed because in case of devices having more than one derivatives
+            #will have diffrent model name strings in device ID, because of which we don't get the common ppd name for search)
+
+            ppd_name = self.mq.get('ppd-name',0)
+    
+            if ppd_name == 0:    #Means ppd-name is not provided So follow earlier path of getting name from device ID.
+            	model = cups.stripModel2(self.model)
+            	self.print_ppd = cups.getPPDFile2(model, self.ppds)
+            else:
+            	self.print_ppd = cups.getPPDFile2(ppd_name, self.ppds)
+            
         finally:
             QApplication.restoreOverrideCursor()
 
@@ -802,14 +823,14 @@ class SetupDialog(QDialog, Ui_Dialog):
         default_model = utils.xstrip(model.replace('series', '').replace('Series', ''), '_')
 
         printer_name = default_model
-
+        installed_printer_names = device.getSupportedCUPSPrinterNames(['hp'])
         # Check for duplicate names
-        if self.device_uri in self.installed_print_devices and \
-            printer_name in self.installed_print_devices[self.device_uri]:
+        if (self.device_uri in self.installed_print_devices and printer_name in self.installed_print_devices[self.device_uri]) \
+           or (printer_name in installed_printer_names):
                 i = 2
                 while True:
                     t = printer_name + "_%d" % i
-                    if t not in self.installed_print_devices[self.device_uri]:
+                    if (t not in installed_printer_names) and (self.device_uri not in self.installed_print_devices or t not in self.installed_print_devices[self.device_uri]):
                         printer_name += "_%d" % i
                         break
                     i += 1
@@ -830,14 +851,14 @@ class SetupDialog(QDialog, Ui_Dialog):
         default_model = utils.xstrip(model.replace('series', '').replace('Series', ''), '_')
 
         fax_name = default_model + "_fax"
-
+        installed_fax_names = device.getSupportedCUPSPrinterNames(['hpfax'])
         # Check for duplicate names
-        if self.fax_uri in self.installed_fax_devices and \
-            fax_name in self.installed_fax_devices[self.fax_uri]:
+        if (self.fax_uri in self.installed_fax_devices and fax_name in self.installed_fax_devices[self.fax_uri]) \
+           or (fax_name in installed_fax_names):
                 i = 2
                 while True:
                     t = fax_name + "_%d" % i
-                    if t not in self.installed_fax_devices[self.fax_uri]:
+                    if (t not in installed_fax_names) and (self.fax_uri not in self.installed_fax_devices or t not in self.installed_fax_devices[self.fax_uri]):
                         fax_name += "_%d" % i
                         break
                     i += 1
@@ -872,8 +893,8 @@ class SetupDialog(QDialog, Ui_Dialog):
                 self.printer_fax_names_same = False
                 self.printer_name_ok = True
 
-                self.FaxNameLineEdit.emit(SIGNAL("textChanged(const QString &)"),
-                            (self.FaxNameLineEdit.text(),))
+                self.FaxNameLineEdit.emit(SIGNAL("textEdited(const QString &)"),
+                            self.FaxNameLineEdit.text())
 
         self.setIndicators()
         self.setAddPrinterButton()
@@ -904,8 +925,8 @@ class SetupDialog(QDialog, Ui_Dialog):
                 self.printer_fax_names_same = False
                 self.fax_name_ok = True
 
-                self.PrinterNameLineEdit.emit(SIGNAL("textChanged(const QString&)"),
-                            (self.PrinterNameLineEdit.text(),))
+                self.PrinterNameLineEdit.emit(SIGNAL("textEdited(const QString&)"),
+                            self.PrinterNameLineEdit.text())
 
         self.setIndicators()
         self.setAddPrinterButton()
@@ -985,11 +1006,10 @@ class SetupDialog(QDialog, Ui_Dialog):
 
                 QApplication.restoreOverrideCursor()
                 if os.geteuid!=0 and utils.addgroup()!=[]:
-                    FailureUI(self, self.__tr("<b>Printer queue setup failed.</b><p>Please add user to %s group(s)" %utils.list_to_string(utils.addgroup())))
+                    FailureUI(self, self.__tr("<b>Printer queue setup failed. Could not connect to CUPS Server</b><p>Is user added to %s group(s)" %utils.list_to_string(utils.addgroup())))
             else:
-                # TODO:
-                #service.sendEvent(self.hpssd_sock, EVENT_CUPS_QUEUES_CHANGED, device_uri=self.device_uri)
-                pass
+                # sending Event to add this device in hp-systray
+                utils.sendEvent(EVENT_CUPS_QUEUES_CHANGED,self.device_uri, self.printer_name)
 
         finally:
             QApplication.restoreOverrideCursor()
@@ -1017,10 +1037,9 @@ class SetupDialog(QDialog, Ui_Dialog):
                 QApplication.restoreOverrideCursor()
                 FailureUI(self, self.__tr("<b>Fax queue setup failed.</b><p>Please restart CUPS and try again."))
             else:
-                pass
-                # TODO:
-                #service.sendEvent(self.hpssd_sock, EVENT_CUPS_QUEUES_CHANGED, device_uri=self.fax_uri)
-
+                 # sending Event to add this device in hp-systray
+                utils.sendEvent(EVENT_CUPS_QUEUES_CHANGED,self.fax_uri, self.fax_name)
+                
         finally:
             QApplication.restoreOverrideCursor()
 
@@ -1253,7 +1272,7 @@ class SetupDialog(QDialog, Ui_Dialog):
                     printer = unicode(item.data(Qt.UserRole).toString()).encode('utf-8')
                     log.debug("Removing printer: %s" % printer)
                     if cups.delPrinter(printer) == 0 and os.geteuid!=0 and utils.addgroup()!=[]:
-                            FailureUI(self, self.__tr("<b>Unable to delete printer queue.</b><p>Please add user to %s group(s)" %utils.list_to_string(utils.addgroup())))
+                            FailureUI(self, self.__tr("<b>Unable to delete printer queue. Could not connect to CUPS Server</b><p>Is user added to %s group(s)" %utils.list_to_string(utils.addgroup())))
             self.close()
 
         else:
